import 'chat_manager.dart';
import 'internal/chat_method_keys.dart';
import 'internal/chat_channel_manager.dart';
import 'models/chat_error.dart';
import 'models/chat_message.dart';
import 'models/chat_translate_language.dart';

extension ChatTransformPlugin on ChatManager {
  ///
  /// Translate a message.
  ///
  /// Param [msg] The message object
  ///
  /// Param [languages] The target languages to translate
  ///
  /// **Return** Translated Message
  ///
  /// **Throws**  A description of the exception. See {@link ChatError}.
  ///
  Future<ChatMessage> translateMessage({
    required ChatMessage msg,
    required List<String> languages,
  }) async {
    Map req = {};
    req["message"] = msg.toJson();
    req["languages"] = languages;
    Map result = await ChatMethodChannel.ChatManager.invokeMethod(
        ChatMethodKeys.translateMessage, req);
    try {
      ChatError.hasErrorFromResult(result);
      return ChatMessage.fromJson(result["message"]);
    } on ChatError catch (e) {
      throw e;
    }
  }

  ///
  /// Fetch all languages what the translate service support
  ///
  /// **Return** Supported languages
  ///
  /// **Throws**  A description of the exception. See {@link ChatError}.
  ///
  Future<List<ChatTranslateLanguage>> fetchSupportedLanguages() async {
    Map result = await ChatMethodChannel.ChatManager.invokeMethod(
        ChatMethodKeys.fetchSupportLanguages);
    try {
      ChatError.hasErrorFromResult(result);
      List<ChatTranslateLanguage> list = [];
      result[ChatMethodKeys.fetchSupportLanguages]?.forEach((element) {
        list.add(ChatTranslateLanguage.fromJson(element));
      });
      return list;
    } on ChatError catch (e) {
      throw e;
    }
  }
}
