import '../tools/chat_extension.dart';

import 'chat_enums.dart';
import 'chat_message_body.dart';

///
/// The command message body.
///
class ChatCmdMessageBody extends ChatMessageBody {
  ///
  /// Creates a command message.
  ///
  ChatCmdMessageBody({required this.action, this.deliverOnlineOnly = false})
      : super(type: MessageType.CMD);

  /// @nodoc
  ChatCmdMessageBody.fromJson({required Map map})
      : super.fromJson(map: map, type: MessageType.CMD) {
    this.action = map["action"];
    this.deliverOnlineOnly =
        map.getBoolValue("deliverOnlineOnly", defaultValue: false)!;
  }

  /// @nodoc
  @override
  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = super.toJson();
    data.setValueWithOutNull("action", action);
    data.setValueWithOutNull("deliverOnlineOnly", deliverOnlineOnly);

    return data;
  }

  /// The command action content.
  late final String action;

  ///
  /// Checks whether this command message is only delivered to online users.
  ///
  /// - `true`: Yes.
  /// - `false`: No.
  ///
  bool deliverOnlineOnly = false;
}
